# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from .....core.api_error import ApiError
from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.jsonable_encoder import jsonable_encoder
from .....core.pydantic_utilities import pydantic_v1
from .....core.request_options import RequestOptions
from ....commons.errors.entity_not_found_error import EntityNotFoundError
from ....commons.errors.http_request_validation_error import HttpRequestValidationError
from ....commons.errors.updates_disabled_due_to_external_system_integration_error import (
    UpdatesDisabledDueToExternalSystemIntegrationError,
)
from ....commons.types.entity_not_found_error_message import EntityNotFoundErrorMessage
from ....commons.types.page_token import PageToken
from ....commons.types.request_validation_error import RequestValidationError
from ....commons.types.updates_disabled_due_to_external_system_integration_error_message import (
    UpdatesDisabledDueToExternalSystemIntegrationErrorMessage,
)
from ..v_2.types.organization_provider_id import OrganizationProviderId
from ..v_2.types.organization_provider_sort_options import OrganizationProviderSortOptions
from .types.organization_provider_create_v_2 import OrganizationProviderCreateV2
from .types.organization_provider_page_v_2 import OrganizationProviderPageV2
from .types.organization_provider_update_v_2 import OrganizationProviderUpdateV2
from .types.organization_provider_v_2 import OrganizationProviderV2

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class V3Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get(
        self,
        organization_provider_id: OrganizationProviderId,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> OrganizationProviderV2:
        """
        Parameters
        ----------
        organization_provider_id : OrganizationProviderId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        OrganizationProviderV2

        Examples
        --------
        import uuid

        from candid.client import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.organization_providers.v_3.get(
            organization_provider_id=uuid.UUID(
                "965a563a-0285-4910-9569-e3739c0f6eab",
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/organization-providers/v3/{jsonable_encoder(organization_provider_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(OrganizationProviderV2, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    pydantic_v1.parse_obj_as(EntityNotFoundErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_multi(
        self,
        *,
        limit: typing.Optional[int] = None,
        search_term: typing.Optional[str] = None,
        npi: typing.Optional[str] = None,
        is_rendering: typing.Optional[bool] = None,
        is_billing: typing.Optional[bool] = None,
        page_token: typing.Optional[PageToken] = None,
        sort: typing.Optional[OrganizationProviderSortOptions] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> OrganizationProviderPageV2:
        """
        Parameters
        ----------
        limit : typing.Optional[int]
            Limit the number of results returned. Defaults to 100.

        search_term : typing.Optional[str]
            Filter to a name or a part of a name.

        npi : typing.Optional[str]
            Filter to a specific NPI.

        is_rendering : typing.Optional[bool]
            Filter to only rendering providers.

        is_billing : typing.Optional[bool]
            Filter to only billing providers.

        page_token : typing.Optional[PageToken]
            The page token to continue paging through a previous request.

        sort : typing.Optional[OrganizationProviderSortOptions]
            Defaults to PROVIDER_NAME_ASC.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        OrganizationProviderPageV2

        Examples
        --------
        from candid.client import CandidApiClient
        from candid.resources.organization_providers.v_2 import (
            OrganizationProviderSortOptions,
        )

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.organization_providers.v_3.get_multi(
            limit=100,
            search_term="john",
            npi="1234567890",
            is_rendering=True,
            is_billing=True,
            page_token="eyJ0b2tlbiI6IjEiLCJwYWdlX3Rva2VuIjoiMiJ9",
            sort=OrganizationProviderSortOptions.PROVIDER_NAME_ASC,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/organization-providers/v3",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            params={
                "limit": limit,
                "search_term": search_term,
                "npi": npi,
                "is_rendering": is_rendering,
                "is_billing": is_billing,
                "page_token": page_token,
                "sort": sort,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(OrganizationProviderPageV2, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self, *, request: OrganizationProviderCreateV2, request_options: typing.Optional[RequestOptions] = None
    ) -> OrganizationProviderV2:
        """
        Parameters
        ----------
        request : OrganizationProviderCreateV2

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        OrganizationProviderV2

        Examples
        --------
        import datetime

        from candid import (
            DateRangeOptionalEnd,
            IdentifierCode,
            IdentifierCreate,
            IdentifierValue_MedicareProviderIdentifier,
            State,
            StreetAddressLongZip,
        )
        from candid.client import CandidApiClient
        from candid.resources.organization_providers.v_2 import (
            AddressType,
            LicenseType,
            OrganizationProviderAddress,
            ProviderType,
        )
        from candid.resources.organization_providers.v_3 import (
            OrganizationProviderCreateV2,
        )

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.organization_providers.v_3.create(
            request=OrganizationProviderCreateV2(
                npi="string",
                is_rendering=True,
                is_billing=True,
                first_name="string",
                last_name="string",
                organization_name="string",
                provider_type=ProviderType.INDIVIDUAL,
                tax_id="string",
                taxonomy_code="string",
                license_type=LicenseType.MD,
                addresses=[
                    OrganizationProviderAddress(
                        address=StreetAddressLongZip(
                            address_1="123 Main St",
                            address_2="Apt 1",
                            city="New York",
                            state=State.NY,
                            zip_code="10001",
                            zip_plus_four_code="1234",
                        ),
                        address_type=AddressType.DEFAULT,
                    )
                ],
                employment_start_date=datetime.date.fromisoformat(
                    "2023-01-15",
                ),
                employment_termination_date=datetime.date.fromisoformat(
                    "2023-01-15",
                ),
                qualifications=[
                    IdentifierCreate(
                        period=DateRangeOptionalEnd(
                            start_date="string",
                            end_date={"key": "value"},
                        ),
                        identifier_code=IdentifierCode.MCR,
                        identifier_value=IdentifierValue_MedicareProviderIdentifier(),
                    )
                ],
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/organization-providers/v3",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(OrganizationProviderV2, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "HttpRequestValidationError":
                raise HttpRequestValidationError(
                    pydantic_v1.parse_obj_as(RequestValidationError, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "UpdatesDisabledDueToExternalSystemIntegrationError":
                raise UpdatesDisabledDueToExternalSystemIntegrationError(
                    pydantic_v1.parse_obj_as(UpdatesDisabledDueToExternalSystemIntegrationErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update(
        self,
        organization_provider_id: OrganizationProviderId,
        *,
        request: OrganizationProviderUpdateV2,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> OrganizationProviderV2:
        """
        Parameters
        ----------
        organization_provider_id : OrganizationProviderId

        request : OrganizationProviderUpdateV2

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        OrganizationProviderV2

        Examples
        --------
        import uuid

        from candid import (
            DateRangeOptionalEnd,
            IdentifierCode,
            IdentifierValue_MedicareProviderIdentifier,
            State,
            StreetAddressLongZip,
            UpdatableIdentifier_Add,
        )
        from candid.client import CandidApiClient
        from candid.resources.organization_providers.v_2 import (
            AddressType,
            LicenseType,
            OrganizationProviderAddress,
            ProviderType,
        )
        from candid.resources.organization_providers.v_3 import (
            OrganizationProviderUpdateV2,
        )

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.organization_providers.v_3.update(
            organization_provider_id=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
            request=OrganizationProviderUpdateV2(
                npi="string",
                is_rendering=True,
                is_billing=True,
                first_name="string",
                last_name="string",
                organization_name="string",
                provider_type=ProviderType.INDIVIDUAL,
                tax_id="string",
                taxonomy_code="string",
                license_type=LicenseType.MD,
                addresses=[
                    OrganizationProviderAddress(
                        address=StreetAddressLongZip(
                            address_1="123 Main St",
                            address_2="Apt 1",
                            city="New York",
                            state=State.NY,
                            zip_code="10001",
                            zip_plus_four_code="1234",
                        ),
                        address_type=AddressType.DEFAULT,
                    )
                ],
                employment_start_date="string",
                employment_termination_date="string",
                qualifications=[
                    UpdatableIdentifier_Add(
                        period=DateRangeOptionalEnd(
                            start_date="string",
                            end_date={"key": "value"},
                        ),
                        identifier_code=IdentifierCode.MCR,
                        identifier_value=IdentifierValue_MedicareProviderIdentifier(),
                    )
                ],
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/organization-providers/v3/{jsonable_encoder(organization_provider_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="PATCH",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(OrganizationProviderV2, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "HttpRequestValidationError":
                raise HttpRequestValidationError(
                    pydantic_v1.parse_obj_as(RequestValidationError, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    pydantic_v1.parse_obj_as(EntityNotFoundErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "UpdatesDisabledDueToExternalSystemIntegrationError":
                raise UpdatesDisabledDueToExternalSystemIntegrationError(
                    pydantic_v1.parse_obj_as(UpdatesDisabledDueToExternalSystemIntegrationErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncV3Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get(
        self,
        organization_provider_id: OrganizationProviderId,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> OrganizationProviderV2:
        """
        Parameters
        ----------
        organization_provider_id : OrganizationProviderId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        OrganizationProviderV2

        Examples
        --------
        import asyncio
        import uuid

        from candid.client import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.organization_providers.v_3.get(
                organization_provider_id=uuid.UUID(
                    "965a563a-0285-4910-9569-e3739c0f6eab",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/organization-providers/v3/{jsonable_encoder(organization_provider_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(OrganizationProviderV2, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    pydantic_v1.parse_obj_as(EntityNotFoundErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_multi(
        self,
        *,
        limit: typing.Optional[int] = None,
        search_term: typing.Optional[str] = None,
        npi: typing.Optional[str] = None,
        is_rendering: typing.Optional[bool] = None,
        is_billing: typing.Optional[bool] = None,
        page_token: typing.Optional[PageToken] = None,
        sort: typing.Optional[OrganizationProviderSortOptions] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> OrganizationProviderPageV2:
        """
        Parameters
        ----------
        limit : typing.Optional[int]
            Limit the number of results returned. Defaults to 100.

        search_term : typing.Optional[str]
            Filter to a name or a part of a name.

        npi : typing.Optional[str]
            Filter to a specific NPI.

        is_rendering : typing.Optional[bool]
            Filter to only rendering providers.

        is_billing : typing.Optional[bool]
            Filter to only billing providers.

        page_token : typing.Optional[PageToken]
            The page token to continue paging through a previous request.

        sort : typing.Optional[OrganizationProviderSortOptions]
            Defaults to PROVIDER_NAME_ASC.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        OrganizationProviderPageV2

        Examples
        --------
        import asyncio

        from candid.client import AsyncCandidApiClient
        from candid.resources.organization_providers.v_2 import (
            OrganizationProviderSortOptions,
        )

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.organization_providers.v_3.get_multi(
                limit=100,
                search_term="john",
                npi="1234567890",
                is_rendering=True,
                is_billing=True,
                page_token="eyJ0b2tlbiI6IjEiLCJwYWdlX3Rva2VuIjoiMiJ9",
                sort=OrganizationProviderSortOptions.PROVIDER_NAME_ASC,
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/organization-providers/v3",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            params={
                "limit": limit,
                "search_term": search_term,
                "npi": npi,
                "is_rendering": is_rendering,
                "is_billing": is_billing,
                "page_token": page_token,
                "sort": sort,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(OrganizationProviderPageV2, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self, *, request: OrganizationProviderCreateV2, request_options: typing.Optional[RequestOptions] = None
    ) -> OrganizationProviderV2:
        """
        Parameters
        ----------
        request : OrganizationProviderCreateV2

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        OrganizationProviderV2

        Examples
        --------
        import asyncio
        import datetime

        from candid import (
            DateRangeOptionalEnd,
            IdentifierCode,
            IdentifierCreate,
            IdentifierValue_MedicareProviderIdentifier,
            State,
            StreetAddressLongZip,
        )
        from candid.client import AsyncCandidApiClient
        from candid.resources.organization_providers.v_2 import (
            AddressType,
            LicenseType,
            OrganizationProviderAddress,
            ProviderType,
        )
        from candid.resources.organization_providers.v_3 import (
            OrganizationProviderCreateV2,
        )

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.organization_providers.v_3.create(
                request=OrganizationProviderCreateV2(
                    npi="string",
                    is_rendering=True,
                    is_billing=True,
                    first_name="string",
                    last_name="string",
                    organization_name="string",
                    provider_type=ProviderType.INDIVIDUAL,
                    tax_id="string",
                    taxonomy_code="string",
                    license_type=LicenseType.MD,
                    addresses=[
                        OrganizationProviderAddress(
                            address=StreetAddressLongZip(
                                address_1="123 Main St",
                                address_2="Apt 1",
                                city="New York",
                                state=State.NY,
                                zip_code="10001",
                                zip_plus_four_code="1234",
                            ),
                            address_type=AddressType.DEFAULT,
                        )
                    ],
                    employment_start_date=datetime.date.fromisoformat(
                        "2023-01-15",
                    ),
                    employment_termination_date=datetime.date.fromisoformat(
                        "2023-01-15",
                    ),
                    qualifications=[
                        IdentifierCreate(
                            period=DateRangeOptionalEnd(
                                start_date="string",
                                end_date={"key": "value"},
                            ),
                            identifier_code=IdentifierCode.MCR,
                            identifier_value=IdentifierValue_MedicareProviderIdentifier(),
                        )
                    ],
                ),
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/organization-providers/v3",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(OrganizationProviderV2, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "HttpRequestValidationError":
                raise HttpRequestValidationError(
                    pydantic_v1.parse_obj_as(RequestValidationError, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "UpdatesDisabledDueToExternalSystemIntegrationError":
                raise UpdatesDisabledDueToExternalSystemIntegrationError(
                    pydantic_v1.parse_obj_as(UpdatesDisabledDueToExternalSystemIntegrationErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update(
        self,
        organization_provider_id: OrganizationProviderId,
        *,
        request: OrganizationProviderUpdateV2,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> OrganizationProviderV2:
        """
        Parameters
        ----------
        organization_provider_id : OrganizationProviderId

        request : OrganizationProviderUpdateV2

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        OrganizationProviderV2

        Examples
        --------
        import asyncio
        import uuid

        from candid import (
            DateRangeOptionalEnd,
            IdentifierCode,
            IdentifierValue_MedicareProviderIdentifier,
            State,
            StreetAddressLongZip,
            UpdatableIdentifier_Add,
        )
        from candid.client import AsyncCandidApiClient
        from candid.resources.organization_providers.v_2 import (
            AddressType,
            LicenseType,
            OrganizationProviderAddress,
            ProviderType,
        )
        from candid.resources.organization_providers.v_3 import (
            OrganizationProviderUpdateV2,
        )

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.organization_providers.v_3.update(
                organization_provider_id=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
                request=OrganizationProviderUpdateV2(
                    npi="string",
                    is_rendering=True,
                    is_billing=True,
                    first_name="string",
                    last_name="string",
                    organization_name="string",
                    provider_type=ProviderType.INDIVIDUAL,
                    tax_id="string",
                    taxonomy_code="string",
                    license_type=LicenseType.MD,
                    addresses=[
                        OrganizationProviderAddress(
                            address=StreetAddressLongZip(
                                address_1="123 Main St",
                                address_2="Apt 1",
                                city="New York",
                                state=State.NY,
                                zip_code="10001",
                                zip_plus_four_code="1234",
                            ),
                            address_type=AddressType.DEFAULT,
                        )
                    ],
                    employment_start_date="string",
                    employment_termination_date="string",
                    qualifications=[
                        UpdatableIdentifier_Add(
                            period=DateRangeOptionalEnd(
                                start_date="string",
                                end_date={"key": "value"},
                            ),
                            identifier_code=IdentifierCode.MCR,
                            identifier_value=IdentifierValue_MedicareProviderIdentifier(),
                        )
                    ],
                ),
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/organization-providers/v3/{jsonable_encoder(organization_provider_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="PATCH",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(OrganizationProviderV2, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "HttpRequestValidationError":
                raise HttpRequestValidationError(
                    pydantic_v1.parse_obj_as(RequestValidationError, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    pydantic_v1.parse_obj_as(EntityNotFoundErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "UpdatesDisabledDueToExternalSystemIntegrationError":
                raise UpdatesDisabledDueToExternalSystemIntegrationError(
                    pydantic_v1.parse_obj_as(UpdatesDisabledDueToExternalSystemIntegrationErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic

from ......core.datetime_utils import serialize_datetime
from ......core.pydantic_utilities import deep_union_pydantic_dicts
from .....commons.types.facility_type_code import FacilityTypeCode
from .....commons.types.network_type import NetworkType
from .....commons.types.procedure_modifier import ProcedureModifier
from .....commons.types.state import State
from .....organization_providers.resources.v_2.types.license_type import LicenseType
from .....organization_providers.resources.v_2.types.organization_provider_id import OrganizationProviderId
from .....payers.resources.v_3.types.payer_uuid import PayerUuid


class Dimensions(pydantic.BaseModel):
    """
    Dimension values that qualify a rate. For the optional dimensions, a null value signifies "all apply". For set-type dimensions, an empty set signifies "all apply".
    """

    payer_uuid: PayerUuid
    organization_billing_provider_id: OrganizationProviderId
    states: typing.Set[State]
    zip_codes: typing.Set[str]
    license_types: typing.Set[LicenseType]
    facility_type_codes: typing.Set[FacilityTypeCode]
    network_types: typing.Set[NetworkType]
    cpt_code: str
    modifiers: typing.Set[ProcedureModifier]

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class InsuranceTypeCode(str, enum.Enum):
    """
    Code identifying the type of insurance policy within a specific insurance program (X12 008020 Element 1336)
    """

    C_01 = "01"
    """
    Short Term Insurance
    """

    C_12 = "12"
    """
    Medicare Secondary Working Aged Beneficiary or Spouse with Employer Group Health Plan
    """

    C_13 = "13"
    """
    Medicare Secondary End-Stage Renal Disease Beneficiary in the Mandated Coordination Period with an Employer's Group Health Plan
    """

    C_14 = "14"
    """
    Medicare Secondary, No-fault Insurance including Auto is Primary
    """

    C_15 = "15"
    """
    Medicare Secondary Worker's Compensation
    """

    C_16 = "16"
    """
    Medicare Secondary Public Health Service (PHS) or Other Federal Agency
    """

    C_17 = "17"
    """
    Dental
    """

    C_18 = "18"
    """
    Vision
    """

    C_19 = "19"
    """
    Prescription Drugs
    """

    C_41 = "41"
    """
    Medicare Secondary Black Lung
    """

    C_42 = "42"
    """
    Medicare Secondary Veteran's Administration
    """

    C_43 = "43"
    """
    Medicare Secondary Disabled Beneficiary Under Age 65 with Large Group Health Plan (LGHP)
    """

    C_47 = "47"
    """
    Medicare Secondary, Other Liability Insurance is Primary
    """

    CAP = "AP"
    """
    Auto Insurance Policy
    """

    CC_1 = "C1"
    """
    Commercial
    """

    CCO = "CO"
    """
    Consolidated Omnibus Budget Reconciliation Act (COBRA)
    """

    CCP = "CP"
    """
    Medicare Conditionally Primary
    """

    CD = "D"
    """
    Disability - Provides periodic payments to replace income when an insured person is unable to work as a result of illness, injury or disease
    """

    CDB = "DB"
    """
    Disability Benefits
    """

    CE = "E"
    """
    Medicare - Point of Service (POS)
    """

    CEP = "EP"
    """
    Exclusive Provider Organization - Gives subscriber a choice of providers from an approved/contracted payer list; there are fixed dollar co-payments for most covered services in return for using plan providers
    """

    CFF = "FF"
    """
    Family or Friends
    """

    CGP = "GP"
    """
    Group Policy - Two or more people who are part of complete unit who enter into an insurance contract with an insurance company
    """

    CHA = "HA"
    """
    Health Insurance Exchange (HIX) Silver
    """

    CHB = "HB"
    """
    Health Insurance Exchange (HIX) Bronze
    """

    CHD = "HD"
    """
    High Deductible Health Plan
    """

    CHG = "HG"
    """
    Health Insurance Exchange (HIX) Gold
    """

    CHM = "HM"
    """
    Health Maintenance Organization (HMO)
    """

    CHN = "HN"
    """
    Health Maintenance Organization (HMO) - Medicare Risk
    """

    CHP = "HP"
    """
    Health Insurance Exchange (HIX) Platinum
    """

    CHS = "HS"
    """
    Special Low Income Medicare Beneficiary - An individual eligible for Medicare for whom Medicaid pays only Medicare premiums
    """

    CIN = "IN"
    """
    Indemnity - Gives a subscriber the choice to select any provider. Payment is fixed percentage of the cost for covered care after satisfying an annual deductible
    """

    CIP = "IP"
    """
    Individual Policy
    """

    CLC = "LC"
    """
    Long Term Care - Coverage designed to help pay for some or all long term care costs, reducing the risk that a policy-holder would need to deplete all of his or her assets to pay for long term care
    """

    CLD = "LD"
    """
    Long Term Policy
    """

    CLI = "LI"
    """
    Life Insurance
    """

    CLT = "LT"
    """
    Litigation
    """

    CM = "M"
    """
    Medicare Advantage Point of Service (POS) Plan that excludes Part D coverage
    """

    CMA = "MA"
    """
    Medicare Part A
    """

    CMB = "MB"
    """
    Medicare Part B
    """

    CMC = "MC"
    """
    Medicaid - Program of health care services made available to medically indigent and other needy persons, regardless of age, under terms of a 1965 amendment to the U.S. Social Security Act
    """

    CMD = "MD"
    """
    Medicare Part D
    """

    CME = "ME"
    """
    Medicare Advantage Preferred Provider Organization (PPO) Plan that excludes Part D Coverage
    """

    CMF = "MF"
    """
    Medicare Advantage Private Fee for Service
    """

    CMH = "MH"
    """
    Medigap Part A - Health insurance policy intended to cover the non-covered portion of expenses eligible for Medicare Part A reimbursement which must be paid by a Medicare beneficiary for health care services and/or supplies received
    """

    CMI = "MI"
    """
    Medigap Part B - Health insurance policy intended to cover the non-covered portion of expenses eligible for Medicare Part B reimbursement which must be paid by a Medicare beneficiary for health care services and/or supplies received
    """

    CMJ = "MJ"
    """
    Medicare Advantage Health Maintenance Organization (HMO) Plan that includes Part D Coverage
    """

    CMK = "MK"
    """
    Medicare Advantage Health Maintenance Organization (HMO) Risk Plan that includes Part D Coverage
    """

    CML = "ML"
    """
    Medicare Advantage Indemnity Plan that includes Part D Coverage
    """

    CMM = "MM"
    """
    Medicare Advantage Preferred Provider Organization (PPO) Plan that includes Part D Coverage
    """

    CMN = "MN"
    """
    Medicare Advantage Indemnity Plan that excludes Part D coverage
    """

    CMO = "MO"
    """
    Medicare Advantage Point of Service (POS) Plan that includes Part D Coverage
    """

    CMP = "MP"
    """
    Medicare Primary - Medicare has the primary responsibility to pay for health care services and/or supplies received by a covered beneficiary (a person entitled to Medicare benefits)
    """

    CMR = "MR"
    """
    Medicare Advantage Health Maintenance Organization (HMO) Risk Plan that excludes Part D coverage
    """

    CMT = "MT"
    """
    Medicare Advantage Health Maintenance Organization (HMO) Plan that excludes Part D coverage
    """

    CMV = "MV"
    """
    Medicare Advantage
    """

    COA = "OA"
    """
    Open Access Point of Service Plan
    """

    COT = "OT"
    """
    Other
    """

    CPL = "PL"
    """
    Personal
    """

    CPP = "PP"
    """
    Personal Payment (Cash - No Insurance)
    """

    CPR = "PR"
    """
    Preferred Provider Organization (PPO)
    """

    CPS = "PS"
    """
    Point of Service (POS)
    """

    CQM = "QM"
    """
    Qualified Medicare Beneficiary - Coverage for a Medicare eligible individual for whom Medicaid pays only for Medicare premiums, co-insurance, and deductibles
    """

    CRP = "RP"
    """
    Property Insurance - Real
    """

    CSP = "SP"
    """
    Supplemental Policy - An insurance policy intended to cover non-covered charges of another insurance policy
    """

    CTF = "TF"
    """
    Tax Equity Fiscal Responsibility Act (TEFRA)
    """

    CU = "U"
    """
    Multiple Options Health Plan
    """

    CWC = "WC"
    """
    Workers Compensation - Coverage provides medical treatment, rehabilitation, lost wages and related expenses arising from a job related injury or disease
    """

    CWU = "WU"
    """
    Wrap Up Policy - A Workers Compensation Policy written for a specific job site, which will include or cover more than one insured
    """

    def visit(
        self,
        c_01: typing.Callable[[], T_Result],
        c_12: typing.Callable[[], T_Result],
        c_13: typing.Callable[[], T_Result],
        c_14: typing.Callable[[], T_Result],
        c_15: typing.Callable[[], T_Result],
        c_16: typing.Callable[[], T_Result],
        c_17: typing.Callable[[], T_Result],
        c_18: typing.Callable[[], T_Result],
        c_19: typing.Callable[[], T_Result],
        c_41: typing.Callable[[], T_Result],
        c_42: typing.Callable[[], T_Result],
        c_43: typing.Callable[[], T_Result],
        c_47: typing.Callable[[], T_Result],
        cap: typing.Callable[[], T_Result],
        cc_1: typing.Callable[[], T_Result],
        cco: typing.Callable[[], T_Result],
        ccp: typing.Callable[[], T_Result],
        cd: typing.Callable[[], T_Result],
        cdb: typing.Callable[[], T_Result],
        ce: typing.Callable[[], T_Result],
        cep: typing.Callable[[], T_Result],
        cff: typing.Callable[[], T_Result],
        cgp: typing.Callable[[], T_Result],
        cha: typing.Callable[[], T_Result],
        chb: typing.Callable[[], T_Result],
        chd: typing.Callable[[], T_Result],
        chg: typing.Callable[[], T_Result],
        chm: typing.Callable[[], T_Result],
        chn: typing.Callable[[], T_Result],
        chp: typing.Callable[[], T_Result],
        chs: typing.Callable[[], T_Result],
        cin: typing.Callable[[], T_Result],
        cip: typing.Callable[[], T_Result],
        clc: typing.Callable[[], T_Result],
        cld: typing.Callable[[], T_Result],
        cli: typing.Callable[[], T_Result],
        clt: typing.Callable[[], T_Result],
        cm: typing.Callable[[], T_Result],
        cma: typing.Callable[[], T_Result],
        cmb: typing.Callable[[], T_Result],
        cmc: typing.Callable[[], T_Result],
        cmd: typing.Callable[[], T_Result],
        cme: typing.Callable[[], T_Result],
        cmf: typing.Callable[[], T_Result],
        cmh: typing.Callable[[], T_Result],
        cmi: typing.Callable[[], T_Result],
        cmj: typing.Callable[[], T_Result],
        cmk: typing.Callable[[], T_Result],
        cml: typing.Callable[[], T_Result],
        cmm: typing.Callable[[], T_Result],
        cmn: typing.Callable[[], T_Result],
        cmo: typing.Callable[[], T_Result],
        cmp: typing.Callable[[], T_Result],
        cmr: typing.Callable[[], T_Result],
        cmt: typing.Callable[[], T_Result],
        cmv: typing.Callable[[], T_Result],
        coa: typing.Callable[[], T_Result],
        cot: typing.Callable[[], T_Result],
        cpl: typing.Callable[[], T_Result],
        cpp: typing.Callable[[], T_Result],
        cpr: typing.Callable[[], T_Result],
        cps: typing.Callable[[], T_Result],
        cqm: typing.Callable[[], T_Result],
        crp: typing.Callable[[], T_Result],
        csp: typing.Callable[[], T_Result],
        ctf: typing.Callable[[], T_Result],
        cu: typing.Callable[[], T_Result],
        cwc: typing.Callable[[], T_Result],
        cwu: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is InsuranceTypeCode.C_01:
            return c_01()
        if self is InsuranceTypeCode.C_12:
            return c_12()
        if self is InsuranceTypeCode.C_13:
            return c_13()
        if self is InsuranceTypeCode.C_14:
            return c_14()
        if self is InsuranceTypeCode.C_15:
            return c_15()
        if self is InsuranceTypeCode.C_16:
            return c_16()
        if self is InsuranceTypeCode.C_17:
            return c_17()
        if self is InsuranceTypeCode.C_18:
            return c_18()
        if self is InsuranceTypeCode.C_19:
            return c_19()
        if self is InsuranceTypeCode.C_41:
            return c_41()
        if self is InsuranceTypeCode.C_42:
            return c_42()
        if self is InsuranceTypeCode.C_43:
            return c_43()
        if self is InsuranceTypeCode.C_47:
            return c_47()
        if self is InsuranceTypeCode.CAP:
            return cap()
        if self is InsuranceTypeCode.CC_1:
            return cc_1()
        if self is InsuranceTypeCode.CCO:
            return cco()
        if self is InsuranceTypeCode.CCP:
            return ccp()
        if self is InsuranceTypeCode.CD:
            return cd()
        if self is InsuranceTypeCode.CDB:
            return cdb()
        if self is InsuranceTypeCode.CE:
            return ce()
        if self is InsuranceTypeCode.CEP:
            return cep()
        if self is InsuranceTypeCode.CFF:
            return cff()
        if self is InsuranceTypeCode.CGP:
            return cgp()
        if self is InsuranceTypeCode.CHA:
            return cha()
        if self is InsuranceTypeCode.CHB:
            return chb()
        if self is InsuranceTypeCode.CHD:
            return chd()
        if self is InsuranceTypeCode.CHG:
            return chg()
        if self is InsuranceTypeCode.CHM:
            return chm()
        if self is InsuranceTypeCode.CHN:
            return chn()
        if self is InsuranceTypeCode.CHP:
            return chp()
        if self is InsuranceTypeCode.CHS:
            return chs()
        if self is InsuranceTypeCode.CIN:
            return cin()
        if self is InsuranceTypeCode.CIP:
            return cip()
        if self is InsuranceTypeCode.CLC:
            return clc()
        if self is InsuranceTypeCode.CLD:
            return cld()
        if self is InsuranceTypeCode.CLI:
            return cli()
        if self is InsuranceTypeCode.CLT:
            return clt()
        if self is InsuranceTypeCode.CM:
            return cm()
        if self is InsuranceTypeCode.CMA:
            return cma()
        if self is InsuranceTypeCode.CMB:
            return cmb()
        if self is InsuranceTypeCode.CMC:
            return cmc()
        if self is InsuranceTypeCode.CMD:
            return cmd()
        if self is InsuranceTypeCode.CME:
            return cme()
        if self is InsuranceTypeCode.CMF:
            return cmf()
        if self is InsuranceTypeCode.CMH:
            return cmh()
        if self is InsuranceTypeCode.CMI:
            return cmi()
        if self is InsuranceTypeCode.CMJ:
            return cmj()
        if self is InsuranceTypeCode.CMK:
            return cmk()
        if self is InsuranceTypeCode.CML:
            return cml()
        if self is InsuranceTypeCode.CMM:
            return cmm()
        if self is InsuranceTypeCode.CMN:
            return cmn()
        if self is InsuranceTypeCode.CMO:
            return cmo()
        if self is InsuranceTypeCode.CMP:
            return cmp()
        if self is InsuranceTypeCode.CMR:
            return cmr()
        if self is InsuranceTypeCode.CMT:
            return cmt()
        if self is InsuranceTypeCode.CMV:
            return cmv()
        if self is InsuranceTypeCode.COA:
            return coa()
        if self is InsuranceTypeCode.COT:
            return cot()
        if self is InsuranceTypeCode.CPL:
            return cpl()
        if self is InsuranceTypeCode.CPP:
            return cpp()
        if self is InsuranceTypeCode.CPR:
            return cpr()
        if self is InsuranceTypeCode.CPS:
            return cps()
        if self is InsuranceTypeCode.CQM:
            return cqm()
        if self is InsuranceTypeCode.CRP:
            return crp()
        if self is InsuranceTypeCode.CSP:
            return csp()
        if self is InsuranceTypeCode.CTF:
            return ctf()
        if self is InsuranceTypeCode.CU:
            return cu()
        if self is InsuranceTypeCode.CWC:
            return cwc()
        if self is InsuranceTypeCode.CWU:
            return cwu()



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class NameUse(str, enum.Enum):
    USUAL = "USUAL"
    OFFICIAL = "OFFICIAL"
    TEMP = "TEMP"
    NICKNAME = "NICKNAME"
    ANONYMOUS = "ANONYMOUS"
    OLD = "OLD"
    MAIDEN = "MAIDEN"

    def visit(
        self,
        usual: typing.Callable[[], T_Result],
        official: typing.Callable[[], T_Result],
        temp: typing.Callable[[], T_Result],
        nickname: typing.Callable[[], T_Result],
        anonymous: typing.Callable[[], T_Result],
        old: typing.Callable[[], T_Result],
        maiden: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is NameUse.USUAL:
            return usual()
        if self is NameUse.OFFICIAL:
            return official()
        if self is NameUse.TEMP:
            return temp()
        if self is NameUse.NICKNAME:
            return nickname()
        if self is NameUse.ANONYMOUS:
            return anonymous()
        if self is NameUse.OLD:
            return old()
        if self is NameUse.MAIDEN:
            return maiden()



# This file was auto-generated by Fern from our API Definition.

from .types import (
    AppointmentId,
    ClaimAdjustmentGroupCodes,
    ClaimId,
    ClaimSubmissionPayerResponsibilityType,
    Date,
    DateRangeOptionalEnd,
    Decimal,
    DelayReasonCode,
    Email,
    EmrPayerCrosswalk,
    EncounterExternalId,
    EncounterId,
    EntityConflictErrorMessage,
    EntityNotFoundErrorMessage,
    ErrorMessage,
    FacilityTypeCode,
    HttpServiceUnavailableErrorMessage,
    InsuranceTypeCode,
    IntendedSubmissionMedium,
    InvoiceId,
    LinkUrl,
    NetworkType,
    Npi,
    OrganizationId,
    OrganizationNotAuthorizedErrorMessage,
    PageToken,
    PatientExternalId,
    PatientRelationshipToInsuredCodeAll,
    PhoneNumber,
    PhoneNumberType,
    PreEncounterAppointmentId,
    PreEncounterPatientId,
    Primitive,
    ProcedureModifier,
    ProviderId,
    QualifierCode,
    RateId,
    RegionNational,
    RegionStates,
    Regions,
    Regions_National,
    Regions_States,
    RemovableDateRangeOptionalEnd,
    RemovableDateRangeOptionalEnd_DateRange,
    RemovableDateRangeOptionalEnd_Remove,
    RequestValidationError,
    ResourcePage,
    SchemaId,
    ServiceLineId,
    ServiceLineUnits,
    SortDirection,
    SourceOfPaymentCode,
    State,
    StreetAddressBase,
    StreetAddressLongZip,
    StreetAddressShortZip,
    TaskAssignmentId,
    TaskId,
    TaskNoteId,
    TaxId,
    UnauthorizedErrorMessage,
    UnprocessableEntityErrorMessage,
    UpdatesDisabledDueToExternalSystemIntegrationErrorMessage,
    UserId,
    WorkQueueId,
)
from .errors import (
    EntityConflictError,
    EntityNotFoundError,
    HttpRequestValidationError,
    HttpRequestValidationsError,
    HttpServiceUnavailableError,
    OrganizationNotAuthorizedError,
    UnauthorizedError,
    UnprocessableEntityError,
    UpdatesDisabledDueToExternalSystemIntegrationError,
)

__all__ = [
    "AppointmentId",
    "ClaimAdjustmentGroupCodes",
    "ClaimId",
    "ClaimSubmissionPayerResponsibilityType",
    "Date",
    "DateRangeOptionalEnd",
    "Decimal",
    "DelayReasonCode",
    "Email",
    "EmrPayerCrosswalk",
    "EncounterExternalId",
    "EncounterId",
    "EntityConflictError",
    "EntityConflictErrorMessage",
    "EntityNotFoundError",
    "EntityNotFoundErrorMessage",
    "ErrorMessage",
    "FacilityTypeCode",
    "HttpRequestValidationError",
    "HttpRequestValidationsError",
    "HttpServiceUnavailableError",
    "HttpServiceUnavailableErrorMessage",
    "InsuranceTypeCode",
    "IntendedSubmissionMedium",
    "InvoiceId",
    "LinkUrl",
    "NetworkType",
    "Npi",
    "OrganizationId",
    "OrganizationNotAuthorizedError",
    "OrganizationNotAuthorizedErrorMessage",
    "PageToken",
    "PatientExternalId",
    "PatientRelationshipToInsuredCodeAll",
    "PhoneNumber",
    "PhoneNumberType",
    "PreEncounterAppointmentId",
    "PreEncounterPatientId",
    "Primitive",
    "ProcedureModifier",
    "ProviderId",
    "QualifierCode",
    "RateId",
    "RegionNational",
    "RegionStates",
    "Regions",
    "Regions_National",
    "Regions_States",
    "RemovableDateRangeOptionalEnd",
    "RemovableDateRangeOptionalEnd_DateRange",
    "RemovableDateRangeOptionalEnd_Remove",
    "RequestValidationError",
    "ResourcePage",
    "SchemaId",
    "ServiceLineId",
    "ServiceLineUnits",
    "SortDirection",
    "SourceOfPaymentCode",
    "State",
    "StreetAddressBase",
    "StreetAddressLongZip",
    "StreetAddressShortZip",
    "TaskAssignmentId",
    "TaskId",
    "TaskNoteId",
    "TaxId",
    "UnauthorizedError",
    "UnauthorizedErrorMessage",
    "UnprocessableEntityError",
    "UnprocessableEntityErrorMessage",
    "UpdatesDisabledDueToExternalSystemIntegrationError",
    "UpdatesDisabledDueToExternalSystemIntegrationErrorMessage",
    "UserId",
    "WorkQueueId",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic

from ......core.datetime_utils import serialize_datetime
from ......core.pydantic_utilities import deep_union_pydantic_dicts
from .....commons.types.schema_id import SchemaId


class SchemaInstance(pydantic.BaseModel):
    """
    Examples
    --------
    import uuid

    from candid.resources.custom_schemas.v_1 import SchemaInstance

    SchemaInstance(
        schema_id=uuid.UUID(
            "ec096b13-f80a-471d-aaeb-54b021c9d582",
        ),
        content={
            "provider_category": "internist",
            "is_urgent_care": true,
            "bmi": 24.2,
            "age": 38,
        },
    )
    """

    schema_id: SchemaId = pydantic.Field()
    """
    The schema to which the content must adhere.
    """

    content: typing.Dict[str, typing.Any] = pydantic.Field()
    """
    A set of key-value pairs that adhere to the naming and type convention of the schema. Not all keys in the schema must be included, but attaching any key that does not exist in the schema or attaching a key with the incorrect value type will result in errors.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class InvoiceSortField(str, enum.Enum):
    CREATED_AT = "CREATED_AT"
    UPDATED_AT = "UPDATED_AT"
    PATIENT_EXTERNAL_ID = "PATIENT_EXTERNAL_ID"
    NOTE = "NOTE"
    DUE_DATE = "DUE_DATE"
    STATUS = "STATUS"

    def visit(
        self,
        created_at: typing.Callable[[], T_Result],
        updated_at: typing.Callable[[], T_Result],
        patient_external_id: typing.Callable[[], T_Result],
        note: typing.Callable[[], T_Result],
        due_date: typing.Callable[[], T_Result],
        status: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is InvoiceSortField.CREATED_AT:
            return created_at()
        if self is InvoiceSortField.UPDATED_AT:
            return updated_at()
        if self is InvoiceSortField.PATIENT_EXTERNAL_ID:
            return patient_external_id()
        if self is InvoiceSortField.NOTE:
            return note()
        if self is InvoiceSortField.DUE_DATE:
            return due_date()
        if self is InvoiceSortField.STATUS:
            return status()



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class NonInsurancePayerRefundSortField(str, enum.Enum):
    AMOUNT_CENTS = "amount_cents"
    REFUND_TIMESTAMP = "refund_timestamp"

    def visit(
        self, amount_cents: typing.Callable[[], T_Result], refund_timestamp: typing.Callable[[], T_Result]
    ) -> T_Result:
        if self is NonInsurancePayerRefundSortField.AMOUNT_CENTS:
            return amount_cents()
        if self is NonInsurancePayerRefundSortField.REFUND_TIMESTAMP:
            return refund_timestamp()



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic

from ......core.datetime_utils import serialize_datetime
from ......core.pydantic_utilities import deep_union_pydantic_dicts
from .....identifiers.types.identifier import Identifier
from ...v_2.types.license_type import LicenseType
from ...v_2.types.organization_provider_address import OrganizationProviderAddress
from ...v_2.types.organization_provider_id import OrganizationProviderId
from ...v_2.types.provider_type import ProviderType


class OrganizationProviderV2(pydantic.BaseModel):
    """
    Examples
    --------
    import datetime
    import uuid

    from candid import (
        Identifier,
        IdentifierCode,
        IdentifierValue_MedicareProviderIdentifier,
        State,
    )
    from candid.resources.organization_providers.v_2 import (
        LicenseType,
        ProviderType,
    )
    from candid.resources.organization_providers.v_3 import OrganizationProviderV2

    OrganizationProviderV2(
        npi="1234567890",
        is_rendering=True,
        is_billing=True,
        first_name="John",
        last_name="Doe",
        organization_name="Acme Medical",
        provider_type=ProviderType.INDIVIDUAL,
        tax_id="123456789",
        taxonomy_code="1234567890",
        license_type=LicenseType.MD,
        employment_start_date=datetime.date.fromisoformat(
            "2020-10-07",
        ),
        employment_termination_date=datetime.date.fromisoformat(
            "2021-10-07",
        ),
        organization_provider_id=uuid.UUID(
            "965a563a-0285-4910-9569-e3739c0f6eab",
        ),
        qualifications=[
            Identifier(
                identifier_id=uuid.UUID(
                    "123e4567-e89b-12d3-a456-426614174000",
                ),
                identifier_code=IdentifierCode.MCR,
                identifier_value=IdentifierValue_MedicareProviderIdentifier(
                    state=State.CA,
                    provider_number="1234567890",
                ),
            )
        ],
    )
    """

    npi: str = pydantic.Field()
    """
    The NPI of the provider. This must be all digits [0-9] and exactly 10 characters long.
    """

    is_rendering: bool = pydantic.Field()
    """
    Whether the provider can be used to render services.
    """

    is_billing: bool = pydantic.Field()
    """
    Whether the provider can be used to bill services.
    """

    first_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The first name of the provider, if the provider is an individual.
    """

    last_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The last name of the provider, if the provider is an individual.
    """

    organization_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the provider, if the provider is an organization.
    """

    provider_type: ProviderType = pydantic.Field()
    """
    Whether the provider is an individual (NPPES Type 1) or organization (NPPES Type 2) provider.
    """

    tax_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    If the provider has a contract with insurance, this must be the same tax ID given to the payer on an IRS W-9 form completed during contracting.
    """

    taxonomy_code: typing.Optional[str] = pydantic.Field(default=None)
    """
    A code designating classification and specialization.
    """

    license_type: LicenseType = pydantic.Field()
    """
    The type of license that the provider holds.
    """

    addresses: typing.Optional[typing.List[OrganizationProviderAddress]] = pydantic.Field(default=None)
    """
    The addresses associated with this provider.
    """

    employment_start_date: typing.Optional[dt.date] = pydantic.Field(default=None)
    """
    The employment start date for the provider.
    """

    employment_termination_date: typing.Optional[dt.date] = pydantic.Field(default=None)
    """
    The employment termination date for the provider.
    """

    organization_provider_id: OrganizationProviderId = pydantic.Field()
    """
    Auto-generated ID set on creation.
    """

    qualifications: typing.List[Identifier] = pydantic.Field()
    """
    Qualification given to a provider (PTAN, Medicaid Provider Id etc.).
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class EligibilityCheckStatus(str, enum.Enum):
    """
    enum to represent the statuses of eligibility checks
    """

    CREATED = "CREATED"
    COMPLETED = "COMPLETED"
    FAILED = "FAILED"
    PENDING = "PENDING"

    def visit(
        self,
        created: typing.Callable[[], T_Result],
        completed: typing.Callable[[], T_Result],
        failed: typing.Callable[[], T_Result],
        pending: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is EligibilityCheckStatus.CREATED:
            return created()
        if self is EligibilityCheckStatus.COMPLETED:
            return completed()
        if self is EligibilityCheckStatus.FAILED:
            return failed()
        if self is EligibilityCheckStatus.PENDING:
            return pending()



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic

from ......core.datetime_utils import serialize_datetime
from ......core.pydantic_utilities import deep_union_pydantic_dicts
from .....commons.types.decimal import Decimal
from .....commons.types.facility_type_code import FacilityTypeCode
from .....commons.types.procedure_modifier import ProcedureModifier
from .....commons.types.service_line_units import ServiceLineUnits
from .....diagnoses.types.diagnosis_id import DiagnosisId
from .drug_identification import DrugIdentification
from .service_line_denial_reason import ServiceLineDenialReason


class ServiceLineUpdate(pydantic.BaseModel):
    edit_reason: typing.Optional[str] = None
    modifiers: typing.Optional[typing.List[ProcedureModifier]] = None
    charge_amount_cents: typing.Optional[int] = None
    diagnosis_id_zero: typing.Optional[DiagnosisId] = None
    diagnosis_id_one: typing.Optional[DiagnosisId] = None
    diagnosis_id_two: typing.Optional[DiagnosisId] = None
    diagnosis_id_three: typing.Optional[DiagnosisId] = None
    drug_identification: typing.Optional[DrugIdentification] = None
    denial_reason: typing.Optional[ServiceLineDenialReason] = None
    place_of_service_code: typing.Optional[FacilityTypeCode] = None
    units: typing.Optional[ServiceLineUnits] = None
    procedure_code: typing.Optional[str] = None
    quantity: typing.Optional[Decimal] = pydantic.Field(default=None)
    """
    String representation of a Decimal that can be parsed by most libraries.
    A ServiceLine quantity cannot contain more than one digit of precision.
    Example: 1.1 is valid, 1.11 is not.
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    A free-form description to clarify the related data elements and their content. Maps to SV1-01, C003-07 on the 837-P.
    """

    date_of_service: typing.Optional[dt.date] = pydantic.Field(default=None)
    """
    date_of_service must be defined on either the encounter or the service lines but not both.
    """

    end_date_of_service: typing.Optional[dt.date] = None

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from .types import (
    Appointment,
    AppointmentStatus,
    AppointmentWorkQueue,
    MutableAppointment,
    Service,
    UniversalServiceIdentifier,
)

__all__ = [
    "Appointment",
    "AppointmentStatus",
    "AppointmentWorkQueue",
    "MutableAppointment",
    "Service",
    "UniversalServiceIdentifier",
]
